/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache license, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the license for the specific language governing permissions and
 * limitations under the license.
 */
package org.apache.logging.log4j.status;

import java.util.concurrent.atomic.AtomicInteger;

/**
 * This is a dummy class and is only here to allow module-info.java to compile.
 * It will not be copied into the log4j-api module.
 */
public final class StatusLogger {

    private static final StatusLogger STATUS_LOGGER = new StatusLogger();

    public void error(String message, Object p0, Object p1) {
    }

    public void warn(String message, Object p0, Object p1) {
    }

    public static StatusLogger getLogger() {
        return STATUS_LOGGER;
    }
}
