/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.log4j.spi;

/**
 * Implementors decide when to perform an appender specific action.
 * 
 * <p>
 * For example, the {@code org.apache.log4j.net.SMTPAppender} sends an email when the
 * {@link #isTriggeringEvent(LoggingEvent)} method returns {@code true} and adds the event to an internal buffer when
 * the returned result is {@code false}.
 * </p>
 * 
 * @since version 1.0
 */
public interface TriggeringEventEvaluator {

    /**
     * Tests if this the triggering event.
     * 
     * @param event The vent to test.
     * @return Whether this the triggering event.
     */
    public boolean isTriggeringEvent(LoggingEvent event);
}
